'=============================================================================
' Copyright  Micromega Corporation 2002-2005. All rights reserved
'
' File: demo2.bas
'
' This program demonstrates the use of the uM-FPU V2 floating point coprocessor
' with the Comfile PICBASIC microcontroller using an SPI interface. It takes
' temperature readings from a DS1620 digital thermometer, converts them to
' floating point and displays them in degrees Celsius and degrees Fahrenheit.
' It also calculates the minimum and maximum temperatures.
'
' @author Cam Thompson, Micromega Corporation
' @version V2.0 - March 3, 2005
' changes:
'   V2.0 - March 3, 2005
'     - modified for Comfile PICBASIC
'	V1.0 - January 8, 2005
'	  - original version
'=============================================================================

'===============================================================================
'-------------------- uM-FPU V2.0 Comfile PICBASIC definitions -----------------
'===============================================================================

'-------------------- uM-FPU opcodes -------------------------------------------

const	SELECTA		= &h00		' select A register
const	SELECTB		= &h10		' select B register
const	WRITEA		= &h20		' write to register and select A
const	WRITEB		= &h30		' write to register and select B
const	FREAD		= &h40		' read from register
const	FSET		= &h50		' A = REG
const	LSET		= &h50		' A = REG
const	FADD		= &h60		' A = A + REG (float)
const	FSUB		= &h70		' A = A - REG (float)
const	FMUL		= &h80		' A = A * REG (float)
const	FDIV		= &h90		' A = A / REG (float)
const	LADD		= &hA0		' A = A + REG (long)
const	LSUB		= &hB0		' A = A - REG (long)
const	LMUL		= &hC0		' A = A * REG (long)
const	LDIV		= &hD0		' A = A / REG (long)
const	SQRT		= &hE0		' A = sqrt(A)	
const	FLOG		= &hE1		' A = ln(A)
const	FLOG10		= &hE2		' A = log(A)
const	FEXP		= &hE3		' A = e ** A
const	FEXP10		= &hE4		' A = 10 ** A
const	FSIN		= &hE5		' A = sin(A) radians
const	FCOS		= &hE6		' A = cos(A) radians
const	FTAN		= &hE7		' A = tan(A) radians
const	FLOOR		= &hE8		' A = nearest integer <= A
const	CEIL		= &hE9		' A = nearest integer >= A
const	ROUND		= &hEA		' A = nearest integer to A
const	NEGATE		= &hEB		' A = -A
const	FABS		= &hEC		' A = |A|
const	INVERSE		= &hED		' A = 1 / A
const	DEGREES		= &hEE		' A = A / (PI / 180) radians to degrees
const	RADIANS		= &hEF		' A = A * (PI / 180) degreees to radians
const	SYNC		= &hF0		' synchronization
const	FLOAT		= &hF1		' float register 0 and store in register A
const	FIX			= &hF2		' copy register A to reg 0 and fix
const	FCOMPARE	= &hF3		' compare A and B
const	LOADBYTE	= &hF4		' write signed byte to reg 0, convert to float
const	LOADUBYTE	= &hF5		' write unsigned byte to reg 0, convert to float
const	LOADWORD	= &hF6		' write signed word to reg 0, convert to float
const	LOADUWORD	= &hF7		' write unsigned word to reg 0, convert to float
const	READSTR		= &hF8		' read zero terminated string
const	ATOF		= &hF9		' convert ASCII to float, store in A register
const	FTOA		= &hFA		' convert float to ASCII
const	ATOL		= &hFB		' convert ASCII to long, store in A register
const	LTOA		= &hFC		' convert long to ASCII
const	FSTATUS		= &hFD		' get the status of A register
const	XOP			= &hFE		' extended opcode
const	NOP			= &hFF		' nop

const	FUNCTION	= &h00		' (XOP) user functions 0-15
const	READBYTE	= &h90      ' (XOP) read low 8 bits of A (long)
const	READWORD	= &h91      ' (XOP) read low 16 bits of A (long)
const	READLONG	= &h92      ' (XOP) read 32-bit long value from A
const	READFLOAT	= &h93      ' (XOP) read floating point value from A
const	LINCA		= &h94      ' (XOP) A = A + 1 (long)
const	LINCB		= &h95      ' (XOP) A = A + 1 (long)
const	LDECA		= &h96      ' (XOP) A = A - 1 (long)
const	LDECB		= &h97      ' (XOP) A = A - 1 (long)
const	LAND		= &h98      ' (XOP) A = A AND B (long)
const	LOR			= &h99      ' (XOP) A = A OR B (long)
const	LXOR		= &h9A      ' (XOP) A = A XOR B (long)
const	LNOT		= &h9B      ' (XOP) A = NOT A (long)
const	LTST		= &h9C      ' (XOP) status of A AND B (long)
const	LSHIFT		= &h9D      ' (XOP) shift A by B bits (long)
const	LWRITEA		= &hA0		' (XOP) write long to register and select A
const	LWRITEB		= &hB0		' (XOP) write long to register and select B
const	LREAD		= &hC0		' (XOP) read long from register
const	LUDIV		= &hD0		' (XOP) A = A / REG (unsigned long)
const	POWER		= &hE0		' (XOP) A = A ** B
const	ROOT		= &hE1		' (XOP) A = the Bth root of A
const	FMIN		= &hE2		' (XOP) A = minimum of A and B
const	FMAX		= &hE3		' (XOP) A = maximum of A and B
const	FRACTION	= &hE4		' (XOP) load reg 0 with the fractional part of A
const	ASIN		= &hE5		' (XOP) A = asin(A) radians
const	ACOS		= &hE6		' (XOP) A = acos(A) radians
const	ATAN		= &hE7		' (XOP) A = atan(A) radians
const	ATAN2		= &hE8		' (XOP) A = atan(A/B)
const	LCOMPARE	= &hE9		' (XOP) long compare A and B
const	LUCOMPARE	= &hEA		' (XOP) unsigned long compare A and B
const	LSTATUS		= &hEB		' (XOP) long status
const	LNEGATE		= &hEC		' (XOP) A = -A (long)
const	LABS		= &hED		' (XOP) A = |A| (long)
const	LEFT		= &hEE		' (XOP) right parenthesis
const	RIGHT		= &hEF		' (XOP) left parenthesis
const	LOADZERO	= &hF0		' (XOP) load register 0 with zero
const	LOADONE		= &hF1		' (XOP) load register 0 with 1.0
const	LOADE		= &hF2		' (XOP) load register 0 with e
const	LOADPI		= &hF3		' (XOP) load register 0 with pi
const	LONGBYTE	= &hF4		' (XOP) write signed byte to reg 0, convert to long
const	LONGUBYTE	= &hF5		' (XOP) write unsigned byte to reg 0, convert to long
const	LONGWORD	= &hF6		' (XOP) write signed word to reg 0, convert to long
const	LONGUWORD	= &hF7		' (XOP) write unsigned word to reg 0, convert to long
const	IEEEMODE	= &hF8		' (XOP) set IEEE mode (default)
const	PICMODE		= &hF9		' (XOP) set PIC mode
const	CHECKSUM	= &hFA		' (XOP) calculate code checksum
const	DBREAK		= &hFB		' (XOP) debug breakpoint
const	TRACEOFF	= &hFC		' (XOP) turn debug trace off
const	TRACEON		= &hFD		' (XOP) turn debug trace on
const	TRACESTR	= &hFE		' (XOP) send debug string to trace buffer
const	VERSION		= &hFF		' (XOP) get version string

const	SyncChar	= &h5C      ' sync character 

'-------------------- uM-FPU status bits ---------------------------------------

const	IS_ZERO		= 0x81				' positive zero
const	IS_NZERO	= 0x83				' negative zero
const	IS_NEGATIVE	= 0x82				' negative
const	IS_NAN		= 0x84				' NaN (Not-a-Number)
const	IS_PINF		= 0x88				' positive infinity
const	IS_NINF		= 0x8A				' negative infinity

'-------------------- uM-FPU pin definitions -----------------------------------

const	FPU_CLK		= 15				' uM-FPU clock (to CLK pin)
const	FPU_DATOUT	= 14				' uM-FPU data output (to SIN pin)
const	FPU_DATIN	= 14				' uM-FPU data input (to SOUT pin)
const	Terminal	= 12				' serial interface to Terminal

const	Sbaud		= 47				' baud rate constant
const	Smode		= 0					' serial mode (0-normal, 1- inverted)
const	Sint 		= 1					' interval between characters

'-------------------- uM-FPU variable definitions ------------------------------

dim dataByte as byte					' opcode or data byte
dim format as byte						' format byte

'==================== end of uM-FPU definitions ================================

'-------------------- DS1620 pin definitions -----------------------------------

const	DS_RST		= 9					' DS1620 reset/enable
const	DS_CLK		= 10				' DS1620 clock
const	DS_DATA		= 11				' DS1620 data out

'-------------------- uM-FPU register definitions ------------------------------

const	DegC		= 1					' temperature in degrees Celsius
const	DegC_Min	= 2					' minimum temperature
const	DegC_Max	= 3					' maximum temperature
const	DegF		= 4					' temperature in degrees Fahrenheit
const	F1_8		= 5					' constant 1.8
const	F32			= 6					' constant 32.0

'-------------------- variables ------------------------------------------------

dim	rawTemp	as integer					' raw temperature reading
dim reg as byte							' register to display

'===============================================================================
'-------------------- initialization -------------------------------------------
'===============================================================================

setup:
	serout Terminal, Sbaud, Smode, Sint, [13, 10, 13, 10, "Demo 1: "] 
	
	gosub fpu_reset						' reset the uM-FPU

	if dataByte = SyncChar then			' check for synchronization
		gosub print_version				' print the version string
	else
		serout Terminal, Sbaud, Smode, Sint, ["uM-FPU not detected"]
		goto done
	end if

	gosub init_DS1620					' initalize the DS1620

	' load floating point constants
	'------------------------------
	dataByte = F1_8						' select F1_8 as A register
	gosub fpu_sendOp
	dataByte = ATOF						' load ASCII string to uM-FPU,
	gosub fpu_sendOp					'  convert to floating point,
	dataByte = "1"						'  store in register 0
	gosub fpu_sendByte					'  select register 0 as B register
	dataByte = "."
	gosub fpu_sendByte
	dataByte = "8"
	gosub fpu_sendByte
	dataByte = 0
	gosub fpu_sendByte
	dataByte = FSET						' F1_8 = register 0 (1.8)
	gosub fpu_sendOp

	dataByte = F32						' select F32 as A register
	gosub fpu_sendOp
	dataByte = LOADBYTE					' load the value 32 to register 0,
	gosub fpu_sendOp					'  convert to floating point,
	dataByte = 32						'  select register 0 as the B register
	gosub fpu_sendByte
	dataByte = FSET						' F32 = register 0 (32.0)
	gosub fpu_sendOp

	serout Terminal, Sbaud, Smode, Sint, [13, 10, "               Min   Current   Max"]
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "              -----  -------  -----"]

	gosub Read_Temp						' initialize min/max values
	dataByte = DegC_Min					' select DegC_Min as A register
	gosub fpu_sendOp
	dataByte = FSET+DegC				' DegC_Min = DegC
	gosub fpu_sendOp
	dataByte = DegC_Max					' select DegC_Max as A register
	gosub fpu_sendOp
	dataByte = FSET+DegC				' DegC_Min = DegC
	gosub fpu_sendOp

'===============================================================================
'-------------------- main routine ---------------------------------------------
'===============================================================================

Main:
	gosub Read_Temp						' get temperature reading

	dataByte = SELECTB+DegC				' select DegC as B register
	gosub fpu_sendOp
	dataByte = DegC_Min					' select DegC_Min as A register
	gosub fpu_sendOp
	dataByte = FMIN						' set DegC_Min to minimum of A and B
	gosub fpu_sendXop

	dataByte = DegC_Max					' select DegC_Max as A register
	gosub fpu_sendOp
	dataByte = FMAX						' set DegC_Max to maximum of A and B
	gosub fpu_sendXop

   ' display temperatures in Celsius
   '--------------------------------
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "Celsius   "]
	dataByte = DegC_Min
	gosub Print_TempC
	dataByte = DegC
	gosub Print_TempC
	dataByte = DegC_Max
	gosub Print_TempC

   ' display temperatures in Fahrenheit
   '-----------------------------------
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "Fahrenheit"]
	reg = DegC_Min
	gosub Print_TempF
	reg = DegC
	gosub Print_TempF
	reg = DegC_Max
	gosub Print_TempF
	
	serout Terminal, Sbaud, Smode, Sint, [13, 10]
	delay 2000							' delay for 2 seconds
	goto main							' get the next reading

done:									' end of program
endLoop:								' loop forever at end of program
	goto endloop

'-------------------- Read_Temp ------------------------------------------------

Read_Temp:
	gosub Read_DS1620					' read temperature value from DS1620

	' send rawTemp to uM-FPU, convert to floating point, store in register
	'---------------------------------------------------------------------
	dataByte = DegC						' select DegC as A register
	gosub fpu_sendOp
	dataByte = LOADWORD					' load rawTemp to register 0,
	gosub fpu_sendOp					'  convert to floating point,
	dataByte = rawTemp.H				'  select register 0 as B register
	gosub fpu_sendByte
	dataByte = rawTemp.L
	gosub fpu_sendByte
	dataByte = FSET						' DegC = register 0 (i.e. set to the
	gosub fpu_sendOp                    '  floatiing point value of razwTemp)

	' divide the rawTemp by 2 to get degrees Celsius
	'-----------------------------------------------
	dataByte = LOADBYTE					' load the value 2 to register 0,
	gosub fpu_sendOp					'  convert to floating point,
	dataByte = 2						'  select register 0 as the B register
	gosub fpu_sendByte
	dataByte = FDIV						' divide DegC by register 0
	gosub fpu_sendOp					'  (i.e.. divide by 2)
	return
	
'-------------------- Print_TempC ----------------------------------------------

Print_TempC:
 	gosub fpu_sendOp					' select A register

	format = 81							' print formatted floating point value
	gosub Print_FloatFormat
	return

'-------------------- Print_TempF --------------------------------------------

Print_TempF:
	' degF = reg * 1.8 + 32
	'----------------------
	dataByte = degF						' select DegF as A register
	gosub	fpu_sendOp
	dataByte = FSET+reg					' set DegF = reg
	gosub fpu_sendOp
	dataByte = FMUL+F1_8				' multiply DegF by 1.8
	gosub fpu_sendOp
	dataByte = FADD+F32					' add 32.0 to DegF
	gosub fpu_sendOp

	format = 81							' print formatted floating point value
	gosub Print_FloatFormat
	return

'-------------------- init_DS1620 ----------------------------------------------

init_DS1620:
	out DS_RST, 0						' initialize pin states
	out DS_CLK, 1
	delay 100

	out DS_RST, 1						' configure for CPU control
	shiftout DS_CLK, DS_DATA, 0, &h0C, 8
	shiftout DS_CLK, DS_DATA, 0, &h02, 8
	out DS_RST, 0
	delay 100

	out DS_RST, 1						' start temperature conversions
	shiftout DS_CLK, DS_DATA, 0, &hEE, 8
	out DS_RST, 0
	delay 1000							' wait for first conversion
	return

'-------------------- read_DS1620 ----------------------------------------------

read_DS1620:		
	out DS_RST, 1						' read temperature value
	shiftout DS_CLK, DS_DATA, 0, &hAA, 8
	rawTemp = shiftin(DS_CLK, DS_DATA, 0, 9)
	out DS_RST, 0
	return

'===============================================================================
'-------------------- uM-FPU V2.0 Comfile PICBASIC support routines ------------
'===============================================================================
		
fpu_reset:
	out FPU_CLK, 0						' set clock and data lines low
	out FPU_CLK, 0
	out FPU_CLK, 1						' send reset pulse to uM-FPU
	delay 1
	out FPU_CLK, 0
	delay 8								' wait for reset to complete

	dataByte = SYNC						' check for synchronization
	gosub fpu_sendOp					' send SYNC instruction
	goto fpu_readByte					' read the status byte and exit

fpu_wait:
	if in(FPU_DATIN) <> 0 then goto fpu_wait	' wait until uM-FPU is ready
	return
	
fpu_sendXop:
	shiftout FPU_CLK, FPU_DATOUT, 1, XOP, 8		' send XOP prefix

fpu_sendOp:								' send opcode
fpu_sendByte:							' send dataByte
	shiftout FPU_CLK, FPU_DATOUT, 1, dataByte, 8
	return

fpu_readByte:							' read a byte of data
	dataByte = shiftin(FPU_CLK, FPU_DATIN, 1, 8)
	return

print_version:
	dataByte = VERSION					' get the uM-FPU version string
	gosub fpu_sendXop					' send version command
	goto print_str						' display the string
	
print_float:
	format = 0							' set format to 0 (default format)

print_floatFormat:
	dataByte = FTOA						' convert floating point to ASCII
	goto print_format

print_long:
	format = 0							' set format to 0 (default format)

print_longFormat:
	dataByte = LTOA						' convert long integer to ASCII
	goto print_format
	
print_format:
	shiftout FPU_CLK, FPU_DATOUT, 1, dataByte, 8	' send conversion command
	shiftout FPU_CLK, FPU_DATOUT, 1, format, 8		' send format
	
print_str:
	if in(FPU_DATIN) <> 0 then goto print_str		' wait until uM-FPU is ready
	shiftout FPU_CLK, FPU_DATOUT, 1, READSTR, 8		' send command to read string
	
print_str2:
	dataByte = shiftin(FPU_CLK, FPU_DATIN, 1, 8)	' display zero terminated string
	if dataByte = 0 or dataByte > 127 then return
	serout Terminal, Sbaud, Smode, Sint, [dataByte] 
	goto print_str2
	 	
'==================== end of uM-FPU support routines ===========================