'===============================================================================
' Copyright  Micromega Corporation 2004-2005. All rights reserved
'
' File: graph-i2c.bas
'
' This program demonstrates the use of the uM-FPU V2 floating point coprocessor
' with the Comfile PICBASIC microcontroller using an SPI interface. It draws a
' simple graph of the sine and cosine functions on the serial terminal.
'
' @author Cam Thompson, Micromega Corporation
' @version V2.0 - March 3, 2005
' changes:
'   V2.0 - March 3, 2005
'     - modified for Comfile PICBASIC
'	V1.0 - July 5, 2004
'	  - original version
'===============================================================================


'===============================================================================
'-------------------- uM-FPU V2.0 Comfile PICBASIC definitions -----------------
'===============================================================================

'-------------------- uM-FPU opcodes -------------------------------------------

const	SELECTA		= &h00		' select A register
const	SELECTB		= &h10		' select B register
const	WRITEA		= &h20		' write to register and select A
const	WRITEB		= &h30		' write to register and select B
const	FREAD		= &h40		' read from register
const	FSET		= &h50		' A = REG
const	LSET		= &h50		' A = REG
const	FADD		= &h60		' A = A + REG (float)
const	FSUB		= &h70		' A = A - REG (float)
const	FMUL		= &h80		' A = A * REG (float)
const	FDIV		= &h90		' A = A / REG (float)
const	LADD		= &hA0		' A = A + REG (long)
const	LSUB		= &hB0		' A = A - REG (long)
const	LMUL		= &hC0		' A = A * REG (long)
const	LDIV		= &hD0		' A = A / REG (long)
const	SQRT		= &hE0		' A = sqrt(A)	
const	FLOG		= &hE1		' A = ln(A)
const	FLOG10		= &hE2		' A = log(A)
const	FEXP		= &hE3		' A = e ** A
const	FEXP10		= &hE4		' A = 10 ** A
const	FSIN		= &hE5		' A = sin(A) radians
const	FCOS		= &hE6		' A = cos(A) radians
const	FTAN		= &hE7		' A = tan(A) radians
const	FLOOR		= &hE8		' A = nearest integer <= A
const	CEIL		= &hE9		' A = nearest integer >= A
const	ROUND		= &hEA		' A = nearest integer to A
const	NEGATE		= &hEB		' A = -A
const	FABS		= &hEC		' A = |A|
const	INVERSE		= &hED		' A = 1 / A
const	DEGREES		= &hEE		' A = A / (PI / 180) radians to degrees
const	RADIANS		= &hEF		' A = A * (PI / 180) degreees to radians
const	SYNC		= &hF0		' synchronization
const	FLOAT		= &hF1		' float register 0 and store in register A
const	FIX			= &hF2		' copy register A to reg 0 and fix
const	FCOMPARE	= &hF3		' compare A and B
const	LOADBYTE	= &hF4		' write signed byte to reg 0, convert to float
const	LOADUBYTE	= &hF5		' write unsigned byte to reg 0, convert to float
const	LOADWORD	= &hF6		' write signed word to reg 0, convert to float
const	LOADUWORD	= &hF7		' write unsigned word to reg 0, convert to float
const	READSTR		= &hF8		' read zero terminated string
const	ATOF		= &hF9		' convert ASCII to float, store in A register
const	FTOA		= &hFA		' convert float to ASCII
const	ATOL		= &hFB		' convert ASCII to long, store in A register
const	LTOA		= &hFC		' convert long to ASCII
const	FSTATUS		= &hFD		' get the status of A register
const	XOP			= &hFE		' extended opcode
const	NOP			= &hFF		' nop

const	FUNCTION	= &h00		' (XOP) user functions 0-15
const	READBYTE	= &h90      ' (XOP) read low 8 bits of A (long)
const	READWORD	= &h91      ' (XOP) read low 16 bits of A (long)
const	READLONG	= &h92      ' (XOP) read 32-bit long value from A
const	READFLOAT	= &h93      ' (XOP) read floating point value from A
const	LINCA		= &h94      ' (XOP) A = A + 1 (long)
const	LINCB		= &h95      ' (XOP) A = A + 1 (long)
const	LDECA		= &h96      ' (XOP) A = A - 1 (long)
const	LDECB		= &h97      ' (XOP) A = A - 1 (long)
const	LAND		= &h98      ' (XOP) A = A AND B (long)
const	LOR			= &h99      ' (XOP) A = A OR B (long)
const	LXOR		= &h9A      ' (XOP) A = A XOR B (long)
const	LNOT		= &h9B      ' (XOP) A = NOT A (long)
const	LTST		= &h9C      ' (XOP) status of A AND B (long)
const	LSHIFT		= &h9D      ' (XOP) shift A by B bits (long)
const	LWRITEA		= &hA0		' (XOP) write long to register and select A
const	LWRITEB		= &hB0		' (XOP) write long to register and select B
const	LREAD		= &hC0		' (XOP) read long from register
const	LUDIV		= &hD0		' (XOP) A = A / REG (unsigned long)
const	POWER		= &hE0		' (XOP) A = A ** B
const	ROOT		= &hE1		' (XOP) A = the Bth root of A
const	FMIN		= &hE2		' (XOP) A = minimum of A and B
const	FMAX		= &hE3		' (XOP) A = maximum of A and B
const	FRACTION	= &hE4		' (XOP) load reg 0 with the fractional part of A
const	ASIN		= &hE5		' (XOP) A = asin(A) radians
const	ACOS		= &hE6		' (XOP) A = acos(A) radians
const	ATAN		= &hE7		' (XOP) A = atan(A) radians
const	ATAN2		= &hE8		' (XOP) A = atan(A/B)
const	LCOMPARE	= &hE9		' (XOP) long compare A and B
const	LUCOMPARE	= &hEA		' (XOP) unsigned long compare A and B
const	LSTATUS		= &hEB		' (XOP) long status
const	LNEGATE		= &hEC		' (XOP) A = -A (long)
const	LABS		= &hED		' (XOP) A = |A| (long)
const	LEFT		= &hEE		' (XOP) right parenthesis
const	RIGHT		= &hEF		' (XOP) left parenthesis
const	LOADZERO	= &hF0		' (XOP) load register 0 with zero
const	LOADONE		= &hF1		' (XOP) load register 0 with 1.0
const	LOADE		= &hF2		' (XOP) load register 0 with e
const	LOADPI		= &hF3		' (XOP) load register 0 with pi
const	LONGBYTE	= &hF4		' (XOP) write signed byte to reg 0, convert to long
const	LONGUBYTE	= &hF5		' (XOP) write unsigned byte to reg 0, convert to long
const	LONGWORD	= &hF6		' (XOP) write signed word to reg 0, convert to long
const	LONGUWORD	= &hF7		' (XOP) write unsigned word to reg 0, convert to long
const	IEEEMODE	= &hF8		' (XOP) set IEEE mode (default)
const	PICMODE		= &hF9		' (XOP) set PIC mode
const	CHECKSUM	= &hFA		' (XOP) calculate code checksum
const	DBREAK		= &hFB		' (XOP) debug breakpoint
const	TRACEOFF	= &hFC		' (XOP) turn debug trace off
const	TRACEON		= &hFD		' (XOP) turn debug trace on
const	TRACESTR	= &hFE		' (XOP) send debug string to trace buffer
const	VERSION		= &hFF		' (XOP) get version string

const	SyncChar	= &h5C      ' sync character 

'-------------------- uM-FPU status bits ---------------------------------------

const	IS_ZERO		= 0x81				' positive zero
const	IS_NZERO	= 0x83				' negative zero
const	IS_NEGATIVE	= 0x82				' negative
const	IS_NAN		= 0x84				' NaN (Not-a-Number)
const	IS_PINF		= 0x88				' positive infinity
const	IS_NINF		= 0x8A				' negative infinity

'-------------------- uM-FPU pin definitions -----------------------------------

const	FPU_CLK		= 15				' uM-FPU clock (to CLK pin)
const	FPU_DATOUT	= 14				' uM-FPU data output (to SIN pin)
const	FPU_DATIN	= 14				' uM-FPU data input (to SOUT pin)
const	Terminal	= 12				' serial interface to Terminal

const	Sbaud		= 47				' baud rate constant
const	Smode		= 0					' serial mode (0-normal, 1- inverted)
const	Sint 		= 1					' interval between characters

'-------------------- uM-FPU variable definitions ------------------------------

dim dataByte as byte					' opcode or data byte
dim format as byte						' format byte

'==================== end of uM-FPU definitions ================================

'-------------------- uM-FPU register definitions ------------------------------

const	Angle		= 1					' angle
const	Point		= 2					' point on graph

'-------------------- variables ------------------------------------------------

const	Space		= 32				' space " "
const	Asterisk	= 42				' asterisk "*"
const	Vline		= 124				' vertical line "|"

dim cnt as byte							' counter
dim tmp as byte							' temporary value
dim x as byte							' X value for graph
dim p as byte							' point value for graph

'===============================================================================
'-------------------- initialization -------------------------------------------
'===============================================================================

setup:
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "Graph - "]

	gosub fpu_reset						' reset the uM-FPU

	if dataByte = SyncChar then			' check for synchronization
		gosub print_version				' print the version string
	else
		serout Terminal, Sbaud, Smode, Sint, ["uM-FPU not detected"]
		goto done
	end if

'===============================================================================
'-------------------- main routine ---------------------------------------------
'===============================================================================

main:
	'graph sine from 0 TO 360 degrees (2*PI radians)
	'-----------------------------------------------
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "Graph of sine"]
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "-1        0        1"]
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "--------------------", 13, 10]
  
	for cnt = 0 TO 20
	  'Angle = sin((pi * 2 * cnt) / 20)
	  '------------------------------
		gosub fpu_wait					' wait until uM-FPU is ready
		dataByte = Angle				' Angle = pi
		gosub fpu_sendOp
		dataByte = LOADPI
		gosub fpu_sendXop
		dataByte = FSET
		gosub fpu_sendOp

		dataByte = LOADBYTE				' Angle = Angle * (cnt * 2)
		gosub fpu_sendOp
		dataByte = 2 * cnt
		gosub fpu_sendByte		
		dataByte = FMUL
		gosub fpu_sendOp

		dataByte = LOADBYTE				' Angle = Angle / 20
		gosub fpu_sendOp
		dataByte = 20
		gosub fpu_sendByte
		dataByte = FDIV
		gosub fpu_sendOp

		dataByte = FSIN					' Angle = sin(Angle)
		gosub fpu_sendOp

		gosub graph						' graph the result
	next cnt

	'graph cosine from 0 TO 360 degrees (2*PI radians)
	'-------------------------------------------------
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "Graph of cosine"]
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "-1        0        1"]
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "--------------------", 13, 10]
  
	for cnt = 0 TO 20
		gosub fpu_wait					' wait until uM-FPU is ready
		dataByte = Angle				' Angle = pi
		gosub fpu_sendOp
		dataByte = LOADPI
		gosub fpu_sendXop
		dataByte = FSET
		gosub fpu_sendOp

		dataByte = LOADBYTE				' Angle = Angle * (cnt * 2)
		gosub fpu_sendOp
		dataByte = cnt * 2
		gosub fpu_sendByte		
		dataByte = FMUL
		gosub fpu_sendOp

		dataByte = LOADBYTE				' Angle = Angle / 20
		gosub fpu_sendOp
		dataByte = 20
		gosub fpu_sendByte
		dataByte = FDIV
		gosub fpu_sendOp

		dataByte = FCOS					' Angle = cos(Angle)
		gosub fpu_sendOp

		gosub graph						' graph the result
	next cnt

done:									' end of program
	serout Terminal, Sbaud, Smode, Sint, [13, 10, "Done."]

endLoop:								' loop forever at end of program
	goto endloop
	
'-------------------- Graph --------------------------------------------------

graph:
	gosub fpu_wait						' wait until uM-FPU is ready
	
	dataByte = Point					' Point = Angle
	gosub fpu_sendOp
	dataByte = FSET+Angle
	gosub fpu_sendOp
	dataByte = LOADBYTE					'p = Fix(Round(Point * 10))
	gosub fpu_sendOp
	dataByte = 10
	gosub fpu_sendByte
	dataByte = FMUL
	gosub fpu_sendOp
	dataByte = ROUND
	gosub fpu_sendOp
	dataByte = FIX
	gosub fpu_sendOp
	gosub fpu_wait						' wait until uM-FPU is ready
	dataByte = SELECTA
	gosub fpu_sendOp
	dataByte = READBYTE					' read value
	gosub fpu_sendXop
	gosub fpu_readByte
	p = dataByte
	
	p = p + 10							' add offset to midpoint of graph

	for x = 0 TO 20						' plot the point on graph
	  if x = p then goto graph1
	  if x = 10 then goto graph2
	  dataByte = Space
	  goto graph3
graph1:
	  dataByte = Asterisk
	  goto graph3
graph2:
	  dataByte = Vline
graph3:
	serout Terminal, Sbaud, Smode, Sint, [dataByte]
	next x

	serout Terminal, Sbaud, Smode, Sint, [" "]
	dataByte = Angle
	gosub fpu_sendOp
	format = 85							' display Value in 8.5 format
	gosub print_floatFormat
	serout Terminal, Sbaud, Smode, Sint, [13, 10]
	return

'===============================================================================
'-------------------- uM-FPU V2.0 Comfile PICBASIC support routines ------------
'===============================================================================
		
fpu_reset:
	out FPU_CLK, 0						' set clock and data lines low
	out FPU_CLK, 0
	out FPU_CLK, 1						' send reset pulse to uM-FPU
	delay 1
	out FPU_CLK, 0
	delay 8								' wait for reset to complete

	dataByte = SYNC						' check for synchronization
	gosub fpu_sendOp					' send SYNC instruction
	goto fpu_readByte					' read the status byte and exit

fpu_wait:
	if in(FPU_DATIN) <> 0 then goto fpu_wait	' wait until uM-FPU is ready
	return
	
fpu_sendXop:
	shiftout FPU_CLK, FPU_DATOUT, 1, XOP, 8		' send XOP prefix

fpu_sendOp:								' send opcode
fpu_sendByte:							' send dataByte
	shiftout FPU_CLK, FPU_DATOUT, 1, dataByte, 8
	return

fpu_readByte:							' read a byte of data
	dataByte = shiftin(FPU_CLK, FPU_DATIN, 1, 8)
	return

print_version:
	dataByte = VERSION					' get the uM-FPU version string
	gosub fpu_sendXop					' send version command
	goto print_str						' display the string
	
print_float:
	format = 0							' set format to 0 (default format)

print_floatFormat:
	dataByte = FTOA						' convert floating point to ASCII
	goto print_format

print_long:
	format = 0							' set format to 0 (default format)

print_longFormat:
	dataByte = LTOA						' convert long integer to ASCII
	goto print_format
	
print_format:
	shiftout FPU_CLK, FPU_DATOUT, 1, dataByte, 8	' send conversion command
	shiftout FPU_CLK, FPU_DATOUT, 1, format, 8		' send format
	
print_str:
	if in(FPU_DATIN) <> 0 then goto print_str		' wait until uM-FPU is ready
	shiftout FPU_CLK, FPU_DATOUT, 1, READSTR, 8		' send command to read string
	
print_str2:
	dataByte = shiftin(FPU_CLK, FPU_DATIN, 1, 8)	' display zero terminated string
	if dataByte = 0 or dataByte > 127 then return
	serout Terminal, Sbaud, Smode, Sint, [dataByte] 
	goto print_str2
	 	
'==================== end of uM-FPU support routines ===========================